<?php
/**
 * フックファンクションクラス
 *
 * 各種フック用のファンクション群
 *
 * @package   risoluto
 * @author    Yuta Hayakawa <haya@hyec.org>
 * @license   http://opensource.org/licenses/bsd-license.php new BSD license
 * @copyright (C) 2008-2009 Yuta Hayakawa / All Rights Reserved.
 */

  class RisolutoHooks
  {

    /**
     * hook_first()
     *
     * 最初に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_first()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_first()

    /**
     * hook_beforeInitController()
     *
     * コントローラ初期化前に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_beforeInitController()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_beforeInitController()

    /**
     * hook_afterInitController()
     *
     * コントローラ初期化後に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_afterInitController()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_afterInitController()

    /**
     * hook_beforeStartSession()
     *
     * セッションスタート前に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_beforeStartSession()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_beforeStartSession()

    /**
     * hook_afterStartSession()
     *
     * セッションスタート後に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_afterStartSession()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_afterStartSession()

    /**
     * hook_beforeLoadClass()
     *
     * クラスインスタンスをロードする前に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_beforeLoadClass()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_beforeLoadClass()

    /**
     * hook_afterLoadClass()
     *
     * クラスインスタンスをロードした後に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_afterLoadClass()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_afterLoadClass()

    /**
     * hook_beforeExecuteClass()
     *
     * クラスインスタンスを実行する前に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_beforeExecuteClass()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_beforeExecuteClass()

    /**
     * hook_afterExecuteClass()
     *
     * クラスインスタンスを実行した後に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_afterExecuteClass()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_afterExecuteClass()

    /**
     * hook_beforeCleanController()
     *
     * コントローラクリーニング前に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_beforeCleanController()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_beforeCleanController()

    /**
     * hook_afterCleanController()
     *
     * コントローラクリーニング後に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_afterCleanController()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_afterCleanController()

    /**
     * hook_end()
     *
     * 最後に実行される
     *
     * @param     void nothing
     * @return    void nothing
     */
    public function hook_end()
    {
      // Notes: ここに必要な処理を記述してください
    } // end of function:hook_end()

  } // end of class:RisolutoHooks
?>
